@@ -10,7 +10,7 @@ from Cookie import SimpleCookie
 
 from util import cached_property, make_traceback, obj_from_str, list_obj_from_str
 from config import cfg
-from exceptions import NotFound, Redirect, PermanentRedirect, BasicAuth
+from exceptions import NotFound, BasicAuth
 from routing import auto_routing, add_apps, url4
 from multidict import MultiDict, HeaderDict
 
@@ -260,7 +260,7 @@ class Response(Exception):
         
         return status, list(self.headers.iterallitems()), [self.body]
         
-            
+
 class App(object):
     def __init__(self, cfg_module='cfg'):
         '''
@@ -269,10 +269,11 @@ class App(object):
         self.cfg_module = cfg_module
 
 
-    def setup(self):
+    def __call__(self, environ, start_response):
         '''
         Первый запуск
         '''
+        setattr(self.__class__, '__call__', self._call)
         def routing_queue(routing):
             list_obj_from_str(routing)
             def wrapper(rq):
@@ -296,33 +297,32 @@ class App(object):
         add_apps(cfg.INSTALLED_APPS)
         
         list_obj_from_str(cfg.CONTEXT_PROCESSORS)
-        list_obj_from_str(cfg.REQUEST_MIDDLEWARES)
-        list_obj_from_str(cfg.RESPONSE_MIDDLEWARES)
+        mdls = list_obj_from_str(cfg.MIDDLEWARES)
+        self.request_middlewares = [m().process_request for m in mdls
+            if hasattr(m, 'process_request')]
+        self.response_middlewares = [m().process_response for m in mdls
+            if hasattr(m, 'process_response')]
+        self.response_middlewares.reverse()
 
         self.routing = obj_from_str(cfg.ROUTING)
-        if isinstance(self.routing, tuple) or isinstance(self.routing, list):
+        if isinstance(self.routing, (tuple, list)):
             self.routing = routing_queue(self.routing)
+            
+        return self._call(environ, start_response)
 
-        self.Request = Request
-
-    def __call__(self, environ, start_response):
+    def _call(self, environ, start_response):
+        rq = Request(environ)
         try:
-            rq = self.Request(environ)
-        except AttributeError:
-            # первый запуск
-            self.setup()
-            rq = self.Request(environ)
-        try:
-            for middleware in cfg.REQUEST_MIDDLEWARES:
+            for middleware in self.request_middlewares:
                 middleware(rq)
             res = self.routing(rq)
             if not isinstance(res, Response):
                 res = Response(unicode(res), content_type='text/plain')
-            for middleware in cfg.RESPONSE_MIDDLEWARES:
+            for middleware in self.response_middlewares:
                 middleware(rq, res)
         except Response, e:
             res = e
-        except (Redirect, PermanentRedirect, BasicAuth), e:
+        except BasicAuth, e:
             res = e.res
         except NotFound, e:
             if cfg.DEBUG:
